@@ -1,7 +1,7 @@
 /* GNU tar Archive Format description.
 
-   Copyright 1988-1989, 1991-1997, 2000-2001, 2003-2007, 2012-2014, 2016
-   Free Software Foundation, Inc.
+   Copyright 1988-1989, 1991-1997, 2000-2001, 2003-2007, 2012-2014,
+   2016-2017 Free Software Foundation, Inc.
 
    This file is part of GNU tar.
 
@@ -328,9 +328,13 @@ struct tar_stat_info
   struct sp_array *sparse_map;
 
   off_t real_size;          /* The real size of sparse file */
-  int   real_size_set;      /* True when GNU.sparse.realsize is set in
+  bool  real_size_set;      /* True when GNU.sparse.realsize is set in
 			       archived file */
 
+  bool  sparse_name_done;   /* Set to true if 'GNU.sparse.name' header was
+                               processed pax header parsing.  Following 'path'
+                               header (lower priority) will be ignored. */
+
   size_t xattr_map_size;   /* Size of the xattr map */
   struct xattr_array *xattr_map;
 
